class TextEditor
  class Command
    class Resolver
      include Component

      def method_missing(method, *args, &block)
        command = resolve(method)
        return super unless command
        command.call(editor, *args, &block)
        self
      end

      def resolve(method)
        return method if method.respond_to?(:call)
        parts = method.to_s.split("_")
        name = parts.map(&:capitalize).join
        command(name)
      end

      private

      def command(name)
        namespace.const_get(name)
      rescue NameError
        nil
      end

      def namespace
        Module.nesting[1]
      end

      def respond_to_missing?(method, include_super)
        resolve(method) || super
      end
    end
  end
end
