class TextEditor
  class Configuration
    class Bootstrap
      include Component

      def configure(args)
        namespace.instance = editor

        options.order!(args)
        options.parse!(args)

        config.files = args.map do |file|
          Pathname.new(file)
        end

        resolver.files.each do |file|
          load file
        end
      end

      def options
        @options ||= OptionParser.new do |opts|
          opts.on("--log FILE", "-l") do |log|
            config.log.file = log
          end

          opts.on("--flavor FLAVOR", "-f") do |flavor|
            config.default_flavor = flavor.to_sym
          end

          opts.on("--version", "-v") do
            puts editor.version
            raise SystemExit
          end
        end
      end

      private

      def resolver
        @resolver ||= Resolver.new(editor)
      end

      def namespace
        Module.nesting.last
      end
    end
  end
end
