class TextEditor
  # An internal representation of a cursor.
  # Each {Window} has it's own cursor instance.
  class Cursor
    attr_reader :line, :column

    def initialize(line = 0, column = 0)
      @line = line
      @column = column
    end

    def clamp(line, column)
      line = [@line, line].min
      column = [@column, column].min
      move(line, column)
    end

    def down(count = 1)
      move(line + count, column)
    end

    def left(count = 1)
      move(line, column - count)
    end

    def move(line, column)
      line = [line, 0].max
      column = [column, 0].max
      self.class.new(line, column)
    end

    def position
      [line, column]
    end

    def right(count = 1)
      move(line, column + count)
    end

    def up(count = 1)
      move(line - count, column)
    end
  end
end
