class TextEditor
  class Keyboard
    include Component
    using CoreExt::Hash

    def aliases
      @aliases ||= {
        "\e" => :escape,
        ctrl_m: :enter,
        dc: :delete,
        ic: :insert,
        npage: :page_down,
        ppage: :page_up,
      }
    end

    def codes
      @codes ||= {
        0 => :ctrl_space,
        9 => :tab,
        27 => :escape,
        127 => :backspace,
        263 => :backspace,
        353 => [:shift, :tab],
        527 => [:ctrl, :shift, :end],
        532 => [:ctrl, :shift, :home],
      }
    end

    def keys
      @keys ||= terminal.keys.transform_keys do |key|
        key.to_s.downcase.sub("key_", "").to_sym
      end
    end

    def listen
      translate(terminal.input)
    end

    def translate(key)
      return unless key

      key = codes[key] if codes.key?(key)
      key = keys.key(key) if keys.value?(key)
      key = aliases[key] if aliases.key?(key)

      case key
      when 0..255 then key.chr.to_sym
      when Array then key
      else key.to_sym
      end
    end
  end
end
