class TextEditor
  # The main event loop for the text editor.
  class Reactor
    include Component

    def running?
      !!@running
    end

    def start
      @running = true
      render
      listen
    end

    def stop
      @running = false
    end

    private

    def listen
      while running?
        key = editor.keyboard.listen
        next unless key

        keybinding, args = editor.mode.resolve(key)
        resolved = command.resolve(keybinding)
        next unless resolved

        resolved.call(editor, *args, key.to_s)
        render
      end
    end

    def render
      editor.window.render
    end
  end
end
