class TextEditor
  class Window
    extend Forwardable
    include Component

    def_delegators :state, :buffer, :cursor

    def cache
      @cache ||= {}
    end

    def file
      config.files.first
    end

    def render
      buffer.each_line.with_index do |line, index|
        if cache[index] != line
          cache[index] = line
          move(index, 0)
          write(line)
        end
      end

      deleted_lines = cache.keys - (0...buffer.size).to_a

      deleted_lines.each do |index|
        cache.delete(index)
        move(index, 0)
        delete_line
      end

      line = cursor.line
      content = buffer.line(line)
      search = content[0..cursor.column]

      tabs = search.scan("\t").size
      width = config.tab.width - 1
      column = cursor.column + (tabs * width)

      move(line, column)
      window.refresh
    end

    def state
      @state || update(Buffer.new(file), Cursor.new)
    end

    def update(buffer, cursor)
      @state = State.new(buffer, cursor)
    end

    private

    def delete_line
      window.deleteln
    end

    def move(*coords)
      window.setpos(*coords)
    end

    def window
      @window ||= terminal.window(terminal.lines, terminal.columns, 0, 0)
    end

    def write(string)
      tabbed = string.gsub("\t", config.tab.to_s)
      padded = tabbed.ljust(terminal.columns)
      window.addstr(padded)
    end
  end
end
